.spec {
    height: 100px;
    width: 100px;
}

.content-table {
    font-size: 14;
    margin-left: auto;
    margin-right: auto;
    text-align: center;
}

.reacnum {
    color: blue;
    margin-top: 20px;
}

.reacid {
    line-height: 100px;
    width: 20px;
}

.plus {
    line-height: 100px;
    width: 10px;
    font-weight: bold;
}

.reacabcdnone {
    display: none;
}

#canvassec * {
    touch-action: none;
}

.imgbg {
    background-image: url(./fire.png)!important;
}

.grid {
    background: -webkit-linear-gradient(top, transparent 44px, blue 45px), -webkit-linear-gradient(left, transparent 44px, blue 45px);
    background-size: 45px 45px;
}

.savesvgarea {
    position: absolute;
    right: 0;
    bottom: 0;
}

.button {
    border: none;
    color: white;
    padding: 15px 32px;
    text-align: center;
    text-decoration: none;
    display: inline-block;
    font-size: 16px;
}

.savesvgbutton {
    background-color: #008CBA;
}

.clearbutton {
    background-color: tomato;
}

.paginationjs-pages {
    display: inline-block!important;
    float: none!important;
}

.narrow {
    text-align:start;
    line-height:100%;
}

.fileUpload input.realbutton {
    position:absolute;
    top: -1000px;
}

.bootstrap-select>.dropdown-toggle.bs-placeholder{
    height: 38px;
}
